(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define('@nonoll/code-snippet/functions', ['exports'], factory) :
  (global = global || self, factory((global.nonoll = global.nonoll || {}, global.nonoll['code-snippet'] = global.nonoll['code-snippet'] || {}, global.nonoll['code-snippet'].functions = {})));
}(this, (function (exports) { 'use strict';

  /**
   * debounce
    <iframe
      src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-debounce.html&module=%2Ffunctions-debounce.html&theme=dark"
      style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
      title="@nonoll/code-snippet"
      allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
      sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
    ></iframe>
   * @memberof module:functions
   * @function
   * @see https://css-tricks.com/debouncing-throttling-explained-examples/
   * @param {TypeVoidFunction} fn
   * @param {Number} [delay=0]
   * @returns {TypeVoidFunction}
   * @example
  import { debounce } from '@nonoll/code-snippet/functions';

  const forExample = () => {
    const delay = 1000 * 1;
    const listener = debounce(res => console.log(res), delay);

    listener('1');
    listener('2');
    listener('3');
    listener(`4 ${new Date()}`);

    setTimeout(() => {
      listener(`5 ${new Date()}`);
    }, delay);
  }

  forExample();
   */
  var debounce = function (fn, /* istanbul ignore next */ delay) {
      if (delay === void 0) { delay = 0; }
      var timer;
      return function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          window.clearTimeout(timer);
          timer = window.setTimeout(function () {
              fn.apply(null, args);
          }, delay);
      };
  };

  /**
   * deepFreeze
   * @param {any} target
   * @memberof module:functions
   * @function
   */
  var deepFreeze = function (target) {
      if (Object.isFrozen(target)) {
          return target;
      }
      if (!Array.isArray(target) && Object.getPrototypeOf(target) !== Object.getPrototypeOf({})) {
          if (Object.getPrototypeOf(target) !== Object.getPrototypeOf(Function)) {
              throw new Error('not allow types');
          }
      }
      Object.freeze(target);
      Object.keys(target).forEach(function (key) { return deepFreeze(target[key]); });
      return target;
  };

  /**
   * empty function
   * @memberof module:functions
   * @alias noop
   */
  function noop() { }

  /**
   * throttle
    <iframe
      src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-throttle.html&module=%2Ffunctions-throttle.html&theme=dark"
      style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
      title="@nonoll/code-snippet"
      allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
      sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
    ></iframe>
   * @memberof module:functions
   * @function
   * @see https://css-tricks.com/debouncing-throttling-explained-examples/
   * @param {TypeVoidFunction} fn
   * @param {Number} [delay=0]
   * @returns {TypeVoidFunction}
   * @example
  import { debounce } from '@nonoll/code-snippet/functions';

  const forExample = () => {
    const delay = 1000 * 1;
    const listener = throttle(res => console.log(res), delay);

    listener(`1 ${new Date()}`);
    listener('2');
    listener('3');
    listener('4');

    setTimeout(() => {
      listener(`5 ${new Date()}`);
    }, delay);
  }

  forExample();
   */
  var throttle = function (fn, /* istanbul ignore next */ delay) {
      if (delay === void 0) { delay = 0; }
      var caller = function (args) {
          fn.apply(null, args);
          base = +new Date();
      };
      var base = null;
      var isLeading = true;
      var stamp = null;
      return function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          if (isLeading) {
              caller(args);
              isLeading = false;
              return;
          }
          stamp = +new Date();
          /* istanbul ignore next */
          base = base || stamp;
          if ((stamp - base) >= delay) {
              /* istanbul ignore next */
              caller(args);
          }
      };
  };

  exports.debounce = debounce;
  exports.deepFreeze = deepFreeze;
  exports.noop = noop;
  exports.throttle = throttle;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=nonoll-code-snippet-functions.umd.js.map
